package com.school.project.model;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.school.project.model.userRegistration.User;
import com.sun.istack.NotNull;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.ToString;
import org.hibernate.annotations.Fetch;
import org.hibernate.annotations.FetchMode;

import javax.persistence.*;
import java.util.Set;

@Data
@Entity()
@NoArgsConstructor
@Table(name = "Student")
@EqualsAndHashCode(exclude = {"courseLists"})
@ToString(exclude = {"courseLists","studentAppUser"})
public class Student {

    @Id
    @Column(name = "student_id")
    @GeneratedValue(strategy = GenerationType.SEQUENCE)
    private Long studentId;

    @NotNull
    @Column(name = "roll_no", unique = true)
    private String rollNumber;

    @NotNull
    @Column(name = "name")
    private String name;

    @NotNull
    @Column(name = "age")
    private Long age;

    @JsonIgnore
    @ManyToMany(fetch = FetchType.LAZY, cascade = {CascadeType.ALL})
    @Fetch(value= FetchMode.SELECT)
    @JoinTable(
            name = "studentsCourses",
            joinColumns = {@JoinColumn(name = "student_id")},
            inverseJoinColumns = {@JoinColumn(name = "course_id")}
    )
    private Set<Course> courseLists;

    @OneToOne(fetch = FetchType.LAZY)
    @MapsId
    @JoinColumn(name = "student_id")
    @JsonIgnore
    private User studentAppUser;
}
